@@ -93,6 +93,13 @@ class SocialAuthBackend(ModelBackend):
                 user = User.objects.create_user(username=username, email=email)
             social_user = self.associate_auth(user, uid, response, details)
         else:
+            # This account was registered to another user, so we raise an
+            # error in such case and the view should decide what to do on
+            # at this moment, merging account is not an option because that
+            # would imply update user references on other apps, that's too
+            # much intrusive
+            if 'user' in kwargs and kwargs['user'] != social_user.user:
+                raise ValueError('Account already in use.')
             user = social_user.user
 
         # Update user account data.
@@ -101,7 +108,7 @@ class SocialAuthBackend(ModelBackend):
         # Update extra_data storage, unless disabled by setting
         if getattr(settings, 'SOCIAL_AUTH_EXTRA_DATA', True):
             extra_data = self.extra_data(user, uid, response, details)
-            if extra_data:
+            if extra_data and social_user.extra_data != extra_data:
                 social_user.extra_data = extra_data
                 social_user.save()
 
@@ -226,7 +233,7 @@ class OpenIDBackend(SocialAuthBackend):
         # Use Attribute Exchange attributes if provided
         resp = ax.FetchResponse.fromSuccessResponse(response)
         if resp:
-            values.update((alias.replace('old_', ''), resp.getSingle(src))
+            values.update((alias.replace('old_', ''), resp.getSingle(src, ''))
                             for src, alias in OLD_AX_ATTRS + AX_SCHEMA_ATTRS)
 
         fullname = values.get('fullname') or ''
@@ -253,6 +260,7 @@ class BaseAuth(object):
     and implement needed methods"""
     def __init__(self, request, redirect):
         self.request = request
+        self.data = request.POST if request.method == 'POST' else request.GET
         self.redirect = redirect
 
     def auth_url(self):
@@ -273,6 +281,11 @@ class BaseAuth(object):
         otherwise return false."""
         return True
 
+    @classmethod
+    def enabled(cls):
+        """Return backend enabled status, all enabled by default"""
+        return True
+
 
 class OpenIdAuth(BaseAuth):
     """
@@ -299,7 +312,7 @@ class OpenIdAuth(BaseAuth):
                                          form_tag_attrs=form_tag)
 
     def auth_complete(self, *args, **kwargs):
-        response = self.consumer().complete(dict(self.request.REQUEST.items()),
+        response = self.consumer().complete(dict(self.data.items()),
                                             self.request.build_absolute_uri())
         if not response:
             raise ValueError('This is an OpenID relying party endpoint')
@@ -363,10 +376,9 @@ class OpenIdAuth(BaseAuth):
         """Return service provider URL.
         This base class is generic accepting a POST parameter that specifies
         provider URL."""
-        if self.request.method != 'POST' or \
-           OPENID_ID_FIELD not in self.request.POST:
+        if OPENID_ID_FIELD not in self.data:
             raise ValueError('Missing openid identifier')
-        return self.request.POST[OPENID_ID_FIELD]
+        return self.data[OPENID_ID_FIELD]
 
 
 class BaseOAuth(BaseAuth):
@@ -393,6 +405,8 @@ class ConsumerBasedOAuth(BaseOAuth):
     ACCESS_TOKEN_URL = ''
     SERVER_URL = ''
     AUTH_BACKEND = None
+    SETTINGS_KEY_NAME = ''
+    SETTINGS_SECRET_NAME = ''
 
     def auth_url(self):
         """Returns redirect url"""
@@ -409,7 +423,7 @@ class ConsumerBasedOAuth(BaseOAuth):
             raise ValueError('Missing unauthorized token')
 
         token = OAuthToken.from_string(unauthed_token)
-        if token.key != self.request.GET.get('oauth_token', 'no-token'):
+        if token.key != self.data.get('oauth_token', 'no-token'):
             raise ValueError('Incorrect tokens')
 
         access_token = self.access_token(token)
@@ -432,8 +446,8 @@ class ConsumerBasedOAuth(BaseOAuth):
         if extra_params:
             params.update(extra_params)
 
-        if 'oauth_verifier' in self.request.GET:
-            params['oauth_verifier'] = self.request.GET['oauth_verifier']
+        if 'oauth_verifier' in self.data:
+            params['oauth_verifier'] = self.data['oauth_verifier']
         request = OAuthRequest.from_consumer_and_token(self.consumer,
                                                        token=token,
                                                        http_url=url,
@@ -477,9 +491,17 @@ class ConsumerBasedOAuth(BaseOAuth):
 
     def get_key_and_secret(self):
         """Return tuple with Consumer Key and Consumer Secret for current
-        service provider. Must return (key, secret), order must be respected.
+        service provider. Must return (key, secret), order *must* be respected.
         """
-        raise NotImplementedError('Implement in subclass')
+        return getattr(settings, self.SETTINGS_KEY_NAME), \
+               getattr(settings, self.SETTINGS_SECRET_NAME)
+
+    @classmethod
+    def enabled(cls):
+        """Return backend enabled status by checking basic settings"""
+        return all(hasattr(settings, name) for name in
+                        (cls.SETTINGS_KEY_NAME,
+                         cls.SETTINGS_SECRET_NAME))
 
 
 # import sources from where check for auth backends
@@ -502,7 +524,10 @@ def get_backends():
                 try:
                     name = basename(name).replace('.py', '')
                     sub = import_module(mod_name + '.' + name)
-                    backends.update(sub.BACKENDS)
+                    # register only enabled backends
+                    backends.update(((key, val)
+                                        for key, val in sub.BACKENDS.items()
+                                            if val.enabled()))
                 except (ImportError, AttributeError):
                     pass
     return backends
